# DownMan API

TODO: description of api project

REST Api je napsané v programovacím jazyce **[Java](https://java.com)** ve verzi 8 s pomocí **[Spring Boot](https://spring.io/projects/spring-boot)** frameworku ve verzi 2. Spring Boot je nadstavba nad populárním frameworkem **[Spring](https://spring.io)**. REST Api využívá MVC architekturu, která je ve Springu podporovaná pomocí dodatečného modulu.

## Dependencies

Kompilace celého projektu se provádí nástrojem **[maven](https://maven.apache.org)**, ten je třeba mít nainstalovaný vždy. Maven vyžaduje ke svému běhu **Java JVM** a pro úspěšnou kompilaci i **Java SDK**, oboje je nutné nainstalovat před prvotním spuštěním mavenu.

Závislosti samotného REST Api jsou pak řešené v mavenu s pomocí jeho konfiguračního souboru `pom.xml` v root adresáři aplikace. Při prvotním spuštění na daném projektu si maven stáhne veškeré závislosti pro celý projekt, bez nutnosti zásahu ze strany uživatele.

## Configuration

Konfigurace pro REST Api je umístěna v resources aplikace a to v souboru `src/main/resources/application.properties`. Zde je možné nastavit věci související s interními nastaveními serveru, ale také nastavení týkající se vydávání tokenů, jejich platnosti a invalidace. Samozřejmě je zde také možné nastavit úrovně a styl logování.

## Development

Spuštění Api pro vývojové účely je možné pomocí **maven**u a zadáním následujícího příkazu v konzoli `mvn clean spring-boot:run`. Vývojový api server, pak v celé své kráse běží na adrese [http://localhost:8080](http://localhost:8080). **Swagger** dokumentace je dostupná na adrese [http://localhost:8080/swagger-ui.html](http://localhost:8080/swagger-ui.html).

### Migrations

Migrace jsou v Api řešeny pomocí knihovny **Liquibase** a jejího maven pluginu. Liquibase je integrována se Spring Bootem a migrace jsou prováděny při spuštění celé aplikace. Rozdílové skripty jsou generovány semi-automaticky, vývojář je musí spustit ručně a navíc _liquibase_ není schopná vygenerovat všechny změny.

Nastavení migrací je umístěno v souboru `src/main/resources/liquibase.properties`. Zde je nutné znovu nastavit databázové připojení, protože _liquibase_ nepoužívá konfiguraci ze Springu.

Migrace je možné najít ve složce `src/main/resources/db/changelog/` a jsou generované pomocí Liquibase maven pluginu. Vývojář generuje migrace ručně spuštěním příkazu `mvn clean package && mvn liquibase:diff`. Tento příkaz vygeneruje rozdílový skript (aktuální stav databáze oproti Hibernate entitám) v `changelog` složce, který by měl vývojář zkontrolovat a případně doupravit. Pozor na to, že provedení `mvn package` je kritické, bez něj `liquibase:diff` nebude správně fungovat!

Vývojář musí mít na paměti, že `liquibase:diff` není samospásné a nedokáže generovat vše. Z určitých důvodů uvedených v [této issue](https://github.com/liquibase/liquibase-hibernate/issues/48) nedokáže `liquibase-hibernate` generovat změny v datovém typu již existující entity. Z tohoto důvodu je třeba psát migrace datových typů ručně do nově založeného _changeset_ souboru ve složce `src/main/resources/db/changelog`. Tento nově založený soubor by měl následovat nastolenou konvenci a pořadí provádění. Dokumentace liqubase formátu je možné najít na [této adrese](http://www.liquibase.org/documentation/changes/index.html).

## Deployment

Vzhledem k tomu, že Api používá **Spring Boot**, tak se jeho deployment sestává ze dvou kroků a není k němu třeba aplikační server. Build aplikace se dělá pomocí aplikace **maven** a spuštění příkazu `mvn clean package`, tento příkaz zbuildí aplikaci do **JAR** souboru, který je umístěn v `target/download-manager-api.jar`. Tento JAR je pak spustitelný a obsahuje embedded **Tomcat** aplikační server. Samotné produkční spuštění aplikace se dělá pomocí **JVM** a příkazu `java -jar target/download-manager-api.jar`. Samotné spuštění na výsledném fyzickém serveru by mělo být spravováno správcem služeb dostupným v operačním systému nasazeném na serveru.
