package cz.neit.dm.bootstrap;

import cz.neit.dm.entity.*;
import cz.neit.dm.entity.enums.Language;
import cz.neit.dm.entity.enums.MembershipType;
import cz.neit.dm.repositories.*;
import cz.neit.dm.security.acl.Roles;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;

@Slf4j
@Component
public class BootStrapPostgreSQL implements ApplicationListener<ContextRefreshedEvent> {

    private final CategoryRepository categoryRepository;
    private final ClientRepository clientRepository;
    private final GroupRepository groupRepository;
    private final UserRepository userRepository;
    private final DownloadHistoryRepository downloadHistoryRepository;
    private final FileRepository fileRepository;


    public BootStrapPostgreSQL(
            CategoryRepository categoryRepository,
            GroupRepository groupRepository,
            ClientRepository clientRepository,
            UserRepository userRepository,
            DownloadHistoryRepository downloadHistoryRepository,
            FileRepository fileRepository) {
        this.categoryRepository = categoryRepository;
        this.groupRepository = groupRepository;
        this.clientRepository = clientRepository;
        this.userRepository = userRepository;
        this.downloadHistoryRepository = downloadHistoryRepository;
        this.fileRepository = fileRepository;
    }

    @Override
    public void onApplicationEvent(ContextRefreshedEvent event) {

        fillDatabaseWithTestData();

    }

    private void fillDatabaseWithTestData(){
        if (groupRepository.count() != 0L) {
            return;
        }

        log.debug("Filling database with fake data.");

        ////////// GROUPS

        Group group1 = new Group();
        Group group2 = new Group();
        Group group3 = new Group();
        group1.setName("TAS");
        group2.setName("Java");
        group3.setName("Java-cloud");
        group1.setDescription("Team Assistant group");
        group2.setDescription("Java team main group");
        group3.setDescription("Java group for cloud projects");
        groupRepository.save(group1);
        groupRepository.save(group2);
        groupRepository.save(group3);

        ////////// RECIPIENTS

        Recipient recipient1 = new Recipient();
        recipient1.setName("Jan Novák");
        recipient1.setEmail("barouskalf@email.cz");
        clientRepository.save(recipient1);

        Recipient recipient2 = new Recipient();
        recipient2.setName("Jan Novák");
        recipient2.setEmail("hruboskobcb3@gmail.com");
        clientRepository.save(recipient2);

        Recipient recipient3 = new Recipient();
        recipient3.setName("Xandr");
        recipient3.setEmail("nonexisting@test-domain.com");
        clientRepository.save(recipient3);

        ////////// CATEGORIES

        Category category1 = new Category();
        category1.setName("tas-v1");
        Category category2 = new Category();
        category2.setName("java-cloudstorage");
        categoryRepository.save(category1);
        categoryRepository.save(category2);

        ////////// USERS

        User user1 = new User();
        user1.setFirstname("Petr");
        user1.setLastname("Mladý");
        user1.setEmail("petrmlady@neit.eu");
        user1.setPassword("$2a$12$o0o8LS214fC4rpgUaUyf9enw0C5sWOBJjCwza3uqGHag6x5ZsqkUi");
        user1.setLanguage(Language.CZECH);
        user1.setFullname(user1.getFirstname(), user1.getLastname());
        user1.setUsername("petrmlady");
        user1.addGroup(group1, MembershipType.USER);
        user1.addGroup(group2, MembershipType.USER);
        user1.setRole(Roles.ADMINISTRATOR);

        User user2 = new User();
        user2.setFirstname("Jaroslav");
        user2.setLastname("Novák");
        user2.setEmail("novakjarda@neit.eu");
        user2.setPassword("$2a$12$o0o8LS214fC4rpgUaUyf9enw0C5sWOBJjCwza3uqGHag6x5ZsqkUi");
        user2.setLanguage(Language.CZECH);
        user2.setFullname(user2.getFirstname(), user2.getLastname());
        user2.setUsername("jardanovak");
        user2.addGroup(group2, MembershipType.MANAGER);
        user2.setRole(Roles.USER);

        User user3 = new User();
        user3.setFirstname("Alexandr");
        user3.setLastname("Veliký");
        user3.setEmail("hlavnispravce@neit.eu");
        user3.setPassword("$2a$12$o0o8LS214fC4rpgUaUyf9enw0C5sWOBJjCwza3uqGHag6x5ZsqkUi");
        user3.setLanguage(Language.CZECH);
        user3.setFullname(user3.getFirstname(), user3.getLastname());
        user3.setUsername("alexbig");
        user3.addGroup(group1, MembershipType.MANAGER);
        user3.addGroup(group2, MembershipType.MANAGER);
        user3.addGroup(group3, MembershipType.MANAGER);
        user3.setRole(Roles.USER);

        User user4 = new User();
        user4.setFirstname("DeFauLt");
        user4.setLastname("LogGeR");
        user4.setEmail("userforupload@log.eu");
        user4.setPassword("assfsd5f62sd4f5sd");
        user4.setLanguage(Language.CZECH);
        user4.setFullname(user4.getFirstname(), user4.getLastname());
        user4.setUsername("defaultLogUser");
        user4.addGroup(group1, MembershipType.USER);
        user4.addGroup(group2, MembershipType.USER);
        user4.addGroup(group3, MembershipType.USER);
        user4.setRole(Roles.ADMINISTRATOR);

        User user5 = new User();
        user5.setFirstname("Nada");
        user5.setLastname("Horazdovska");
        user5.setEmail("userforupload@log.eu");
        user5.setPassword("$2a$12$PJ74t7x8ebrwldDSFZhrse0pDqAq68zrRDsAqtmNV15dLCvJuiFmu");
        user5.setLanguage(Language.CZECH);
        user5.setFullname(user4.getFirstname(), user4.getLastname());
        user5.setUsername("nadahora");
        user5.setRole(Roles.USER);

        userRepository.save(user1);
        userRepository.save(user2);
        userRepository.save(user3);
        userRepository.save(user4);
        userRepository.save(user5);

        ////////// DELETE DATES

        DeleteDate deleteDate = new DeleteDate();
        deleteDate.setNeverDelete(false);
        deleteDate.setDaysAfterLastShare(null);
        deleteDate.setWillBeDeletedAt(LocalDate.now().plusDays(300));

        ////////// FILES

        FileDM file1 = new FileDM();
        file1.setFilename("filename.ext");
        file1.setFilesize(123456L);
        file1.setFormat("ext");
        file1.setUploadedAt(LocalDate.now());
        file1.setSharedUpTo(null);
        file1.setLastDownloadViaLink(null);
        file1.setDescription("file for testing purposes");
        file1.setOnlyAuthorCanDownload(false);
        file1.setCategory(category1);
        file1.setDeleteDate(deleteDate);
        file1.setGroup(group1);
        file1.setAuthor(user4);

        fileRepository.save(file1);

        ////////// DOWNLOAD HISTORY

        for (int i = 0; i < 5; ++i) {
            DownloadHistory downloadHistory = new DownloadHistory();
            downloadHistory.setDownloadedAt(LocalDateTime.now());
            downloadHistory.setFile(file1);
            downloadHistory.setUser(user2);
            downloadHistory.setSideLink(null);
            downloadHistory.setLink(null);
            downloadHistory.setFileName(file1.getFilename());

            downloadHistoryRepository.save(downloadHistory);
        }
    }
}
