package cz.neit.dm.config;

import cz.neit.dm.io.FileManager;
import cz.neit.dm.services.FileService;
import cz.neit.dm.services.LinkService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.nio.file.Path;

/**
 * Created by jhruby on 25.01.2019
 */

@Slf4j
@Configuration
@EnableScheduling
public class SpringConfig {

    private final FileService fileService;
    private final LinkService linkService;
    private final FileManager fileManager;

    public SpringConfig(FileService fileService, LinkService linkService, FileManager fileManager) {
        this.fileService = fileService;
        this.linkService = linkService;
        this.fileManager = fileManager;
    }

    @Scheduled(cron = "${cron.expression.file}")
    public void scheduleCronDeletingFiles() {
        log.debug("Checking of deprecated files started.");

        fileService.getDeprecatedFiles().forEach(fileDM -> {
            log.debug("Going to delete deprecated file on id: " + fileDM.getId());
            Path path = fileManager.getFilePath(fileService.getFileDetails(fileDM.getId()));
            fileService.deleteFile(fileDM.getId(),path);
        });
    }

    @Scheduled(cron = "${cron.expression.link}")
    public void scheduleCronDisablingDeprecatedLinks() {
        linkService.disableDeprecatedLinks();
    }
}
