package cz.neit.dm.controllers;

import cz.neit.dm.dtos.mappers.UserMapper;
import cz.neit.dm.dtos.modeldto.DefaultUserSettingsDTO;
import cz.neit.dm.dtos.modeldto.UserDTO;
import cz.neit.dm.dtos.modeldto.UserSimpleDTO;
import cz.neit.dm.dtos.modeldto.UserWithPermissionsDTO;
import cz.neit.dm.security.SecurityContextUtils;
import cz.neit.dm.services.DefaultUserSettingsService;
import cz.neit.security.permission.hints.simple.StaticHintEvaluator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * Created by jhruby on 16.04.2019
 */

@Slf4j
@RestController
@RequestMapping("user")
public class DefaultUserSettingsController {

    private final DefaultUserSettingsService defaultUserSettingsService;
    private final UserMapper userMapper;
    private final StaticHintEvaluator staticHintEvaluator;

    public DefaultUserSettingsController(DefaultUserSettingsService defaultUserSettingsService, UserMapper userMapper, StaticHintEvaluator staticHintEvaluator) {
        this.defaultUserSettingsService = defaultUserSettingsService;
        this.userMapper = userMapper;
        this.staticHintEvaluator = staticHintEvaluator;
    }

    @PutMapping("")
    @PreAuthorize("hasPermission('user', 'updateCurrentUser')")
    public UserSimpleDTO updateCurrentUser(@Valid @RequestBody UserDTO userDTO){
        return defaultUserSettingsService.updateCurrentUser(SecurityContextUtils.getCurrentLoggedUser().getId(),userDTO);
    }

    /**
     * This method return information about currently logged user.
     * If JWT token authentication disable this method will be return anonymous UserDTO.
     * @return UserDTO with information about logged user.
     */
    @GetMapping("")
    //TODO preAuthorize
    public UserWithPermissionsDTO getCurrentUser() {
        return new UserWithPermissionsDTO(
                userMapper.userToUserDTO(SecurityContextUtils.getCurrentLoggedUser()),
                staticHintEvaluator.buildStaticHints());
    }

    @PostMapping("")
    @PreAuthorize("hasPermission('user', 'updateCurrentUser')")
    public DefaultUserSettingsDTO setDefaultUserSettings(@Valid @RequestBody DefaultUserSettingsDTO dto){
        return defaultUserSettingsService.setDefaultUserSettings(SecurityContextUtils.getCurrentLoggedUser().getId(),dto);
    }

    @GetMapping("default")
    @PreAuthorize("hasPermission('user', 'updateCurrentUser')")
    public DefaultUserSettingsDTO getDefaultUserSettings(){
        return defaultUserSettingsService.getDefaultUserSettingsDetailsByCurrentUser(SecurityContextUtils.getCurrentLoggedUser());
    }


}
