package cz.neit.dm.controllers;

import cz.neit.dm.dtos.mappers.UserMapper;
import cz.neit.dm.dtos.modeldto.LoginDTO;
import cz.neit.dm.dtos.modeldto.UserWithPermissionsDTO;
import cz.neit.dm.entity.User;
import cz.neit.dm.security.SecurityContextUtils;
import cz.neit.security.jwt.authentication.utils.JwtTokenUtils;
import cz.neit.security.permission.hints.simple.StaticHintEvaluator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

@RestController
public class LoginController {

    private final JwtTokenUtils jwtTokenUtils;
    private final AuthenticationManager authenticationManager;
    private final UserMapper userMapper;
    private final StaticHintEvaluator staticHintEvaluator;

    @Autowired
    public LoginController(JwtTokenUtils jwtTokenUtils,
                           AuthenticationManager authenticationManager, UserMapper userMapper, StaticHintEvaluator staticHintEvaluator) {
        this.jwtTokenUtils = jwtTokenUtils;
        this.authenticationManager = authenticationManager;
        this.userMapper = userMapper;
        this.staticHintEvaluator = staticHintEvaluator;
    }

    @PostMapping("/login")
    public UserWithPermissionsDTO login(@Valid @RequestBody LoginDTO loginDTO, HttpServletResponse response) {
        Authentication authentication = authenticationManager.authenticate(
                new UsernamePasswordAuthenticationToken(
                        loginDTO.getUsername(),
                        loginDTO.getPassword()
                )
        );

        SecurityContextHolder.getContext().setAuthentication(authentication);
        jwtTokenUtils.generateAndSetToken(response, String.valueOf(SecurityContextUtils.getCurrentLoggedUser().getId()));
        return new UserWithPermissionsDTO(userMapper.userToSimpleUserDTO(
                (User) authentication.getPrincipal()),staticHintEvaluator.buildStaticHints());
    }
}
