package cz.neit.dm.controllers;

import cz.neit.dm.entity.PasswordResetToken;
import cz.neit.dm.entity.User;
import cz.neit.dm.mail.EmailService;
import cz.neit.dm.services.DefaultUserSettingsService;
import cz.neit.dm.services.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.UUID;

/**
 * Created by jhruby on 22.03.2019
 */

@Slf4j
@RestController
public class ResetPasswdController {

    private final UserService userService;
    private final DefaultUserSettingsService defaultUserSettingsService;
    private final EmailService emailService;

    public ResetPasswdController(UserService userService, DefaultUserSettingsService defaultUserSettingsService, EmailService emailService) {
        this.userService = userService;
        this.defaultUserSettingsService = defaultUserSettingsService;
        this.emailService = emailService;
    }

    @PostMapping("reset-token/{email}")
    public void generateResetToken(@PathVariable String email) {
        User user = userService.getUserByEmail(email);
        String token = UUID.randomUUID().toString();
        log.debug("Token for reset user password generated.");
        emailService.sendResetTokenEmail(user, userService.createPasswordResetToken(user, token));

    }

    @PostMapping("password/change/{token}")
    public void changePassword(@PathVariable String token, @RequestBody String password) {
        PasswordResetToken passwordResetToken = userService.validatePasswordResetToken(token);
        defaultUserSettingsService.updateUserPassword(password, passwordResetToken.getUser());
        userService.deletePasswordResetToken(token);
        log.debug("Delete passwd reset token: " + token);
    }


}
