package cz.neit.dm.controllers;

import cz.neit.dm.dtos.modeldto.UserDTO;
import cz.neit.dm.dtos.modeldto.UserSimpleDTO;
import cz.neit.dm.services.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by jhruby on 25.07.2018
 */

@Slf4j
@RestController
@RequestMapping("users")
public class UserController {

    private final UserService userService;

    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @GetMapping("all")
    @PreAuthorize("hasPermission('user', 'viewAll')")
    public List<UserDTO> getAllUsers() {
        return userService.getAllUsers();
    }

    @GetMapping("all/simple")
    @PreAuthorize("hasPermission('user', 'viewAll')")
    public List<UserSimpleDTO> getAllSimpleUsers() {
        return userService.getAllSimpleUsers();
    }

    @GetMapping("")
    @PreAuthorize("hasPermission('user', 'viewUsersInGroup')")
    public List<UserDTO> getUsersForGroups(
            @RequestParam(value = "fullname", required = false) String fullNameQuery,
            @RequestParam(value = "groupid", required = false) Long groupId) {

        if (fullNameQuery == null || groupId == null) {
            return userService.getAllUsers();
        } else {
            return userService.getUsersByFullNameOnGroupId(fullNameQuery, groupId);
        }
    }

    @GetMapping("{userId}")
    @PreAuthorize("hasPermission('user', 'viewDetail')")
    public UserDTO getUserById(@PathVariable Long userId) {
        return userService.getUserDetailDTO(userId);
    }

    @PostMapping("")
    @PreAuthorize("hasPermission('user', 'create')")
    public UserDTO createNewUser(@Valid @RequestBody UserDTO userDTO) {
        return userService.createUser(userDTO);
    }

    @PutMapping("{userId}")
    @PreAuthorize("hasPermission('user', 'updateOther')")
    public UserSimpleDTO updateUser(@Valid @RequestBody UserSimpleDTO userSimpleDTO,@PathVariable Long userId){
        return userService.updateUser(userId,userSimpleDTO);
    }

    @DeleteMapping("{userId}")
    @PreAuthorize("hasPermission('user', 'delete')")
    public void deleteUser(@PathVariable Long userId) {
        userService.deleteUser(userId);
    }


    @PostMapping("{userId}/author/{authorId}")
    @PreAuthorize("hasPermission('user', 'delete')")
    public UserSimpleDTO changeAuthorForUserFilesWithoutGroup(@PathVariable Long userId, @PathVariable Long authorId) {
       return userService.changeAuthorOnFilesWithoutGroup(userId,authorId);
    }


}