package cz.neit.dm.dtos.modeldto;

import cz.neit.dm.entity.Link;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.*;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;


/**
 * Created by jhruby on 29.08.2018
 */
@Data
@NoArgsConstructor
@ApiModel(description = "Contains information about uploaded file and additional about file's link, author, group and deleteDate")
public class FileDTO {

    @ApiModelProperty(value = "Unique identificator")
    private Long id;

    @ApiModelProperty(value = "Name of uploaded file")
    @NotBlank(message = "Název souboru nesmí být prázdný.")
    @Size(max = 255,  message = "Název souboru nesmí být delší jak 255 znaků.")
    private String filename;

    @ApiModelProperty(value = "User who upload file to app")
    private UserDTO author;

    @ApiModelProperty(value = "Size of uploaded file")
    @Positive
    private Long filesize;

    @ApiModelProperty(value = "Format of file")
    private String format;

    @ApiModelProperty(value = "Date when the file was uploaded")
    @PastOrPresent(message = "Datum nahrání souboru musí být dnešní nebo minulé.")
    private LocalDate uploadedAt;

    @ApiModelProperty(value = "The latest date of sharing (active recipient's links)")
    private LocalDate sharedUpTo;

    @ApiModelProperty(value = "Last download of file via link")
    @PastOrPresent(message = "Poslední datum stažení souboru skrze odkaz musí být dnešní nebo minulé.")
    private LocalDate lastDownloadViaLink;

    @ApiModelProperty(value = "Information about date, when the file will be deleted from server")
    @Valid
    @NotNull(message = "Datum smazání souboru nesmí být prázdné.")
    private DeleteDateDTO deleteDate;

    @ApiModelProperty(value = "Information about date, when the file will be deleted from server, can be tagged 'true' for never delete, or Date when. ")
    private DeleteDateSimpleDTO deleteDateSimple;

    @ApiModelProperty(value = "The group under which the file is uploaded")
    private GroupDTO group;

    @ApiModelProperty(value = "Category for better filtering")
    @NotNull(message = "Kategorie musí být vyplněna.")
    private String category;

    @ApiModelProperty(value = "Description about uploaded file")
    @Size(max = 1000, message = "Popis souboru nesmí být delší jak 1000 znaků")
    private String description;

    @ApiModelProperty(value = "Info about author privacy of the file")
    private boolean onlyAuthorCanDownload;

    //TODO change to LinkDTO
    @ApiModelProperty(value = "Collection of Link, linked to the file")
    private List<Link> links = new ArrayList<>();

}
