package cz.neit.dm.dtos.modeldto;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.*;
import java.time.LocalDate;
import java.util.List;

/**
 * Created by jhruby on 05.09.2018
 */
@Data
@NoArgsConstructor
@ApiModel(description = "Contains all information needed for share to recipients, what will be sent, to who, how many attempts and how long will be link active ")
public class LinkDTO {

    @ApiModelProperty(value = "Unique identificator")
    @Positive
    private Long id;

    @ApiModelProperty(value = "Date when was the link created")
    @PastOrPresent(message = "Datum vytvoření odkazu musí být dnešní nebo minulé.")
    private LocalDate createdAt;

    @ApiModelProperty(value = "Date of last link sharing")
    @PastOrPresent(message = "Datum posledního sdílení odkazu musí být dnešní nebo minulé.")
    private LocalDate lastShare;

    @ApiModelProperty(value = "Link expiration date")
    @Future(message = "Datum platnosti odkazu musí být budoucí.")
    private LocalDate validUntil;

    @ApiModelProperty(value = "Download limit")
    @PositiveOrZero(message = "Limit počtu stažení nesmí být záporný.")
    @NotNull
    private Long downloadsLimit;

    @ApiModelProperty(value = "Number of current downloaded files via the link")
    @PositiveOrZero(message = "Počet stažení nesmí být záporný.")
    private Long downloadsCount;

    @ApiModelProperty(value = "State about link sharing. If true, link is active for recipients")
    private boolean shared;

    @ApiModelProperty(value = "Description of link, used like name of link for file")
    @NotBlank(message = "Popis odkazu nesmí být prázdný.")
    @Size(max = 1000, message = "Popis odkazu nesmí být delší jak 1000 znaků.")
    private String description;

    @ApiModelProperty(value = "Subject for email")
    @NotBlank(message = "Subjekt mailu nesmí být prázdný.")
    @Size(max = 255, message = "Subjekt mailu nesmí být delší jak 255 znaků.")
    private String mailSubject;

    @ApiModelProperty(value = "Text for sent email")
    @NotBlank(message = "Text mailu nesmí být prázdný.")
    @Size(max = 1000, message = "Text mailu nesmí být delší jak 1000 znaků.")
    private String mailText;

    @ApiModelProperty(value = "Author of link")
    private UserDTO author;

    @ApiModelProperty(value = "File which will be shared")
    @NotNull
    private FileDTO file;

    @ApiModelProperty(value = "Collection of recipients")
    @NotNull
    private List<String> recipients;

}
