package cz.neit.dm.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.neit.dm.entity.enums.Language;
import cz.neit.dm.entity.enums.MembershipType;
import cz.neit.security.jwt.authentication.JwtUserDetails;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import java.security.Principal;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name="dm_users")
public class User implements Principal, UserDetails, JwtUserDetails {
   @Id
   @GeneratedValue(strategy = GenerationType.IDENTITY)
   private Long id;

   @NotBlank
   @Column(length = 64, unique = true)
   private String username;

   private String firstname;
   private String lastname;
   private String fullname;

   @Email
   private String email;
   private String password;
   private String oldPassword;
   private String role;

   @Enumerated(value = EnumType.STRING)
   private Language language;

   @OneToOne(cascade = CascadeType.ALL)
   private DefaultUserSettings defaultUserSettings;

   @OneToMany(/*mappedBy = "author"*/)
   private List<FileDM> fileDMS = new ArrayList<>();


   @OneToMany(orphanRemoval = true, cascade = CascadeType.ALL, mappedBy = "user")
   private List<DownloadHistory> downloadHistories = new ArrayList<>();

   @OneToMany(orphanRemoval = true, cascade = CascadeType.ALL)
   @JoinColumn(name = "user_id")
   private List<GroupMembership> groupMemberships = new ArrayList<>();

   public void addGroup(Group group, MembershipType membershipType){
      groupMemberships.add(new GroupMembership(this, group, membershipType));
   }

   public void setFullname(String firstname, String lastname) {
      this.fullname = firstname + " " + lastname;
   }

   @Override
   public String getName() {
      return username;
   }

   /**
    * Based on user roles, return authorities.
    * @return list of newly created authorities
    */
   @JsonIgnore
   public List<GrantedAuthority> getAuthorities() {
      return Collections.singletonList(new SimpleGrantedAuthority(getRole()));
   }

   @Override
   public boolean isAccountNonExpired() {
      return true;
   }

   @Override
   public boolean isAccountNonLocked() {
      return true;
   }

   @Override
   public boolean isCredentialsNonExpired() {
      return true;
   }

   @Override
   public boolean isEnabled() {
      return true;
   }

   @Override
   public Instant getTokenInvalidationThreshold() {
      return null;
   }
}
