package cz.neit.dm.security.acl.conditions;

import cz.neit.dm.entity.FileDM;
import cz.neit.dm.entity.GroupMembership;
import cz.neit.dm.entity.User;
import cz.neit.dm.entity.enums.MembershipType;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * Created by jhruby on 04.12.2018
 */
public class FileConditions {

    public static boolean isMemberOfGroup(UserDetails userDetails, FileDM fileDM) {
        if (!(userDetails instanceof User) ||
                fileDM == null) {
            return false;
        }

        return ((User) userDetails).getGroupMemberships()
                .stream()
                .map(GroupMembership::getGroup)
                .anyMatch(group -> group.getId().equals(fileDM.getGroup().getId()));
    }

    public static boolean isOwner(UserDetails userDetails, FileDM fileDM) {
        if (!(userDetails instanceof User) ||
                fileDM == null) {
            return false;
        }

        return ((User) userDetails).getId().equals(fileDM.getAuthor().getId());
    }


    public static boolean isManager(UserDetails userDetails, FileDM fileDM) {
        if (!(userDetails instanceof User) ||
                fileDM == null) {
            return false;
        }

        GroupMembership groupMembership = ((User) userDetails).getGroupMemberships()
                .stream()
                .filter(g -> g.getGroup().getId().equals(fileDM.getGroup().getId()))
                .findFirst()
                .orElse(null);

        return groupMembership != null && MembershipType.MANAGER.equals(groupMembership.getMembershipType());
    }

    public static boolean canDownload(UserDetails userDetails, FileDM fileDM) {
        if (!(userDetails instanceof User) ||
                fileDM == null) {
            return false;
        }

        if (fileDM.getGroup() == null) {
            if (fileDM.isOnlyAuthorCanDownload()) {
                return isOwner(userDetails, fileDM);
            } else {
                return true;
            }
        } else if (isMemberOfGroup(userDetails, fileDM)) {
            if (fileDM.isOnlyAuthorCanDownload()) {
                return isOwner(userDetails, fileDM);
            } else {
                return true;
            }
        } else {
            return false;
        }
    }
}
