package cz.neit.dm.security.acl.conditions;

import cz.neit.dm.entity.GroupMembership;
import cz.neit.dm.entity.Link;
import cz.neit.dm.entity.User;
import cz.neit.dm.entity.enums.MembershipType;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * Created by jhruby on 11.12.2018
 */
public class LinkConditions {
    public static boolean isMemberOfGroup(UserDetails userDetails, Link link) {
        if (!(userDetails instanceof User) ||
                link == null) {
            return false;
        }
        return ((User) userDetails).getGroupMemberships()
                .stream()
                .map(GroupMembership::getGroup)
                .anyMatch(group -> group.getId()
                        .equals(link.getFile()
                                .getGroup()
                                .getId()));
    }

    public static boolean isOwner(UserDetails userDetails, Link link) {
        if (!(userDetails instanceof User) ||
                link == null) {
            return false;
        }
        return ((User) userDetails).getId().equals(link.getAuthor().getId());
    }



    public static boolean isManager(UserDetails userDetails, Link link) {
        if (!(userDetails instanceof User) ||
                link == null) {
            return false;
        }

        GroupMembership groupMembership = ((User) userDetails).getGroupMemberships()
                .stream()
                .filter(g -> g.getGroup().getId().equals(link.getFile().getGroup().getId()))
                .findFirst()
                .orElse(null);

        return groupMembership != null && MembershipType.MANAGER.equals(groupMembership.getMembershipType());
    }
}
