package cz.neit.dm.services;

import cz.neit.dm.entity.Category;
import cz.neit.dm.exceptions.ResourceNotFoundException;
import cz.neit.dm.repositories.CategoryRepository;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;


/**
 * Created by jhruby on 26.07.2018
 */

@Slf4j
@Service
@RequiredArgsConstructor
public class CategoryServiceImpl implements CategoryService {

    private final CategoryRepository categoryRepository;

    @Override
    public List<String> getAllCategories() {
        log.debug("Getting All Categories");

        List<String> list = new ArrayList<>();

        categoryRepository.findAll().forEach(category -> list.add(category.getName()));
        return list;
    }

    @Override
    public Category getCategoryById(Long id) {
        log.debug("Getting Category by id: " + id);
        return categoryRepository.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("Kategorie nebyla nalezena"));
    }

    public Category getCategoryByName(String name) {
        log.debug("Getting Category by name: " + name);
        return categoryRepository.findCategoryByName(name)
                .orElseThrow(() -> new ResourceNotFoundException("Kategorie nebyla nalezena"));
    }


    @Override
    public Category createIfCategoryNotExistsByName(String category) {

        Optional<Category> cat = categoryRepository.findCategoryByName(category);
        if (cat.isPresent()) {
            return cat.get();
        } else {
            Category newCategory = new Category();
            newCategory.setName(category);
            return saveAndReturnCategory(newCategory);
        }
    }

    private Category saveAndReturnCategory(Category category) {
        log.debug("Saving new category");
        return categoryRepository.save(category);
    }
}
