package cz.neit.dm.services;

import cz.neit.dm.dtos.modeldto.DeleteDateSimpleDTO;
import cz.neit.dm.dtos.modeldto.FileDTO;
import cz.neit.dm.entity.DeleteDate;
import cz.neit.dm.entity.FileDM;
import cz.neit.dm.exceptions.ResourceNotFoundException;
import cz.neit.dm.repositories.DeleteDateRepository;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;


/**
 * Created by jhruby on 22.01.2019
 */

@Slf4j
@Service
@RequiredArgsConstructor
public class DeleteDateServiceImpl implements DeleteDateService {

    private final DeleteDateRepository deleteDateRepository;

    @Override
    public DeleteDate getDeleteDateDetail(Long id) {
        log.debug("Getting DeleteDate by id: " + id);
        return deleteDateRepository.findById(id).orElseThrow(() -> new ResourceNotFoundException("Datum smazání pro soubor nebylo možné najít."));
    }

    @Override
    public DeleteDate updateDeleteDate(FileDTO file) {
        log.debug("Update DeleteDate.");

        DeleteDate deleteDate = getDeleteDateDetail(file.getDeleteDate().getId());
        deleteDate.setNeverDelete(file.getDeleteDate().getNeverDelete());
        deleteDate.setDeleteAfterLimitExceeded(file.getDeleteDate().getDeleteAfterLimitExceeded());
        deleteDate.setDaysAfterLastShare(file.getDeleteDate().getDaysAfterLastShare());
        deleteDate.setWillBeDeletedAt(file.getDeleteDate().getWillBeDeletedAt());

        return deleteDate;
    }

    @Override
    public boolean isFileToDelete(FileDM fileDM) {
        DeleteDateSimpleDTO dto = getDateWhenFileWillBeDeleted(fileDM);

        if (dto.getNeverDelete()) {
            return false;
        }

        return dto.getDayOfDelete().isBefore(LocalDate.now());
    }

    @Override
    public DeleteDateSimpleDTO getDateWhenFileWillBeDeleted(FileDM fileDM) {

        DeleteDate deleteDate = fileDM.getDeleteDate();
        DeleteDateSimpleDTO simpleDTO = new DeleteDateSimpleDTO();
        simpleDTO.setNeverDelete(false);        // need to be here, boolean must be true or false, refactor to boolean from Boolean
        simpleDTO.setDeleteAfterLimitExceeded(false);

        if (!deleteDate.getNeverDelete() && !deleteDate.getDeleteAfterLimitExceeded() ) {
            if (deleteDate.getDaysAfterLastShare() != null){
                if (fileDM.getSharedUpTo() != null) {
                    simpleDTO.setDayOfDelete(fileDM.getSharedUpTo().plusDays(deleteDate.getDaysAfterLastShare()));
                } else {
                    simpleDTO.setNeverDelete(true);
                }
            } else {
                simpleDTO.setDayOfDelete(deleteDate.getWillBeDeletedAt());
            }
        }

        if (deleteDate.getDeleteAfterLimitExceeded()){
            simpleDTO.setDeleteAfterLimitExceeded(true);
        }

        if(deleteDate.getNeverDelete()) {
            simpleDTO.setNeverDelete(true);
        }

        return simpleDTO;
    }
}
