package cz.neit.dm.services;

import cz.neit.dm.dtos.mappers.DownloadHistoryMapper;
import cz.neit.dm.dtos.modeldto.DownloadHistoryDTO;
import cz.neit.dm.entity.DownloadHistory;
import cz.neit.dm.entity.FileDM;
import cz.neit.dm.entity.SideLink;
import cz.neit.dm.entity.User;
import cz.neit.dm.repositories.DownloadHistoryRepository;
import cz.neit.security.acl.AuthorizatorService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by jhruby on 18.09.2018
 */

@Slf4j
@Service
@RequiredArgsConstructor
public class DownloadHistoryServiceImpl implements DownloadHistoryService {

    private final DownloadHistoryRepository downloadHistoryRepository;
    private final DownloadHistoryMapper downloadHistoryMapper;
    private final SideLinkService sideLinkService;
    private final AuthorizatorService authorizatorService;
    private final FileService fileService;

    @Override
    public List<DownloadHistory> getAllDownloadHistory() {
        List<DownloadHistory> downloadHistories = new ArrayList<>();
        downloadHistoryRepository.findAll().forEach(downloadHistories::add);
        return downloadHistories;
    }

    @Override
    public List<DownloadHistoryDTO> getDownloadHistoryForCurrentUser() {
        if (authorizatorService.isAllowed("history", "viewAll")) {
            log.debug("Getting all Download History");
            return mapDownloadHistoryToDTO(getAllDownloadHistory());
        } else {
            log.debug("Getting Download History for current user");

            return mapDownloadHistoryToDTO(downloadHistoryRepository
                    .findAllByFileIdIn(fileService.getAllFileIdsForCurrentUser()));
        }
    }

    private List<DownloadHistoryDTO> mapDownloadHistoryToDTO(List<DownloadHistory> list) {
        return list.stream()
                .map(downloadHistory -> {
                    DownloadHistoryDTO dtos = downloadHistoryMapper.downloadHistoryToDownloadHistoryDTO(downloadHistory);

                    // TODO linky bude potrebovat ale ne required, budou tam isto toho i Prihlseni uzivatele

                    if (downloadHistory.getSideLink() != null) {
                        dtos.setLink(downloadHistory.getSideLink().getLink());
                        dtos.setFile(downloadHistory.getSideLink().getLink().getFile());       //TODO odstranit a v APP to udelat líp
                        dtos.setRecipient(downloadHistory.getSideLink().getRecipient().getEmail());
                    } else if (downloadHistory.getUser() != null){
                        dtos.setRecipient(downloadHistory.getUser().getFullname());
                        dtos.setFile(downloadHistory.getFile());
                    } else {
                        dtos.setRecipient(downloadHistory.getRecipientEmail());
                    }

                    return dtos;
                })
                .collect(Collectors.toList());
    }

    @Override
    public DownloadHistory addNewDownloadHistory(String sideLinkToken) {

        SideLink sideLink = sideLinkService.getSideLinkByToken(sideLinkToken);

        DownloadHistory downloadHistory = new DownloadHistory();
        downloadHistory.setDownloadedAt(LocalDateTime.now());
        downloadHistory.setSideLink(sideLink);
        downloadHistory.setUser(null);
        downloadHistory.setFile(sideLink.getLink().getFile());
        downloadHistory.setFileName(sideLink.getLink().getFile().getFilename());
        downloadHistory.setRecipientEmail(sideLink.getRecipient().getEmail());

        log.debug("Add new History note, email of user who downloads a file: " + downloadHistory.getSideLink().getRecipient().getEmail());
        return downloadHistoryRepository.save(downloadHistory);
    }

    @Override
    public DownloadHistory addNewDownloadHistory(User user, FileDM fileDM) {
        DownloadHistory downloadHistory = new DownloadHistory();
        downloadHistory.setDownloadedAt(LocalDateTime.now());
        downloadHistory.setSideLink(null);
        downloadHistory.setFile(fileDM);
        downloadHistory.setUser(user);
        downloadHistory.setFileName(fileDM.getFilename());

        log.debug("Add new History note, user who download it: " + user.getUsername());
        return downloadHistoryRepository.save(downloadHistory);
    }

}
