package cz.neit.dm.services;


import cz.neit.dm.dtos.modeldto.FileDTO;
import cz.neit.dm.dtos.modeldto.FileSimpleDTO;
import cz.neit.dm.dtos.modeldto.DownloadTokenDTO;
import cz.neit.dm.entity.FileDM;
import cz.neit.dm.entity.Link;
import org.springframework.web.multipart.MultipartFile;

import java.nio.file.Path;
import java.util.List;

/**
 * Created by jhruby on 26.08.2018
 */

public interface FileService {

    // *********** GETTERS LIST
    List<FileDM> getAllFiles();

    List<FileDTO> getAllFilesDTO();

    List<FileDM> getFilesByAuthorId(Long userId);

    List<FileDTO> getFilesDTOByAuthorId(Long userId);

    List<FileDM> getFilesByUserIdWithoutGroup(Long userId);

    List<FileDM> getFilesByUserIdAndGroupId(Long userId, Long groupId);

    List<FileDTO> getAllFilesForCurrentUser();

    List<Long> getAllFileIdsForCurrentUser();

    List<FileDTO> getFilesWithoutGroup();

    List<FileDM> getFilesByGroup(Long groupId);

    // *********** GETTERS
    FileDTO getFileDetailsWithDescription(Long fileId);

    FileDM getFileDetails(Long fileId);

    FileDTO getFileDetailsDTO(Long fileId);

    FileDM getFileByTokenFromAPP(String token);

    DownloadTokenDTO getDownloadTokenForFile(Long fileId);


    // *********** CRUD
    FileDM createFileWithoutLink(FileDTO fileDTO, MultipartFile multipartFile);

    FileDTO updateFile(FileDTO fileDTO, Long fileId);

    void deleteFile(Long fileId, Path path);


    // *********** CONDITIONS FOR ENDPOINTS
    boolean isDownloadable(Long fileId);


    boolean isShared(Long fileId);

    boolean isAuthorHasFilesInGroup(Long userId, Long groupId);

    boolean isAllLimitsExceeded(Long fileId, Link link);

    // *********** SUPPORTIVE METHODS FOR PROPERTIES
    List<FileSimpleDTO> mapFilesByGroupToSimpleDTO(List<FileDM> fileDMList);

    void setLatestActiveLinkDate(FileDM file);

    void setNewLastDownloadViaLimit(Long fileId);

    void addNewLinkToFileDM(Link link);

    void save(FileDM fileDM);

    // *********** SCHEDULING
    List<FileDM> getDeprecatedFiles();

    boolean isAnyFileLinkIncorrect(FileDM fileDM);

}
