package cz.neit.dm.services;

import cz.neit.dm.dtos.mappers.RecipientMapper;
import cz.neit.dm.dtos.modeldto.RecipientDTO;
import cz.neit.dm.entity.Recipient;
import cz.neit.dm.entity.SideLink;
import cz.neit.dm.exceptions.ResourceNotFoundException;
import cz.neit.dm.repositories.RecipientRepository;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by jhruby on 19.09.2018
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class RecipientServiceImpl implements RecipientService {

    private final RecipientRepository recipientRepository;
    private final RecipientMapper recipientMapper;

    // *********** GETTERS LIST
    @Override
    public List<Recipient> getAllRecipients() {
        log.debug("Getting All recipients.");
        List<Recipient> recipientList = new ArrayList<>();
        recipientRepository.findAll().forEach(recipientList::add);

        return recipientList;
    }

    @Override
    public List<RecipientDTO> getAllRecipientsDTO() {
        log.debug("Getting All recipients DTO.");

        return getAllRecipients().stream()
                .map(recipientMapper::recipientToRecipientDTO)
                .collect(Collectors.toList());
    }

    @Override
    public List<RecipientDTO> getRecipientsByEmail(String email) {
        List<Recipient> recipientList = recipientRepository.findAllByEmailLikeIgnoreCase("%"+email+"%");
        return recipientList.stream()
                .map(recipientMapper::recipientToRecipientDTO)
                .collect(Collectors.toList());
    }

    @Override
    public List<Recipient> getRecipientsAndIfNotExistsCreateNew(List<String> recipientEmails) {

        List<Recipient> recipients = new ArrayList<>();

        recipientEmails.forEach(mail -> {
                if (recipientRepository.findByEmail(mail).isPresent()) {
                    log.debug("Recipient with email: " + mail + ", exists.");
                    recipients.add(recipientRepository.findByEmail(mail).get());

                } else {
                    log.debug("Creating new Recipient with email: " + mail);
                    Recipient newRec = new Recipient();
                    newRec.setEmail(mail);
                    recipients.add(recipientRepository.save(newRec));
                }
        });

        return recipients;
    }

    // *********** GETTERS
    @Override
    public Recipient getRecipientDetails(Long recipientId) {
        log.debug("Getting Recipient by id: " + recipientId);
        return recipientRepository.findById(recipientId).orElseThrow(() -> new ResourceNotFoundException("Příjemce nebyl nalezen."));
    }

    // *********** SUPPORTIVE METHODS FOR PROPERTIES
    @Override
    public void addNewSideLinkToRecipient(SideLink sideLink) {
        Recipient recipient = this.getRecipientDetails(sideLink.getRecipient().getId());
        List<SideLink> sideLinkList = recipient.getSideLinks();
        sideLinkList.add(sideLink);
        recipient.setSideLinks(sideLinkList);
        recipientRepository.save(recipient);
    }
}
