package cz.neit.dm.services;

import cz.neit.dm.entity.User;
import cz.neit.dm.repositories.UserRepository;
import cz.neit.security.jwt.authentication.JwtUserRepository;
import cz.neit.security.jwt.authentication.exception.UserNotFoundException;
import lombok.RequiredArgsConstructor;
import org.hibernate.Hibernate;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by jhruby on 12.11.2018
 */
@Service
@Transactional
@RequiredArgsConstructor
public class UserDetailsServiceImpl implements UserDetailsService, JwtUserRepository {

    private final UserRepository userRepository;

    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username){
        User user = userRepository.findByUsername(username)
                .orElseThrow(() -> new UsernameNotFoundException("Uživatel s uživatelským jménem '" + username + "' nebyl nalezen."));
        user.getGroupMemberships().size();
        return user;
    }

    @Override
    public UserDetails findById(String id) {
        Long longId;
        try {
            longId = Long.parseLong(id);
        } catch (Exception e) {
            throw new UserNotFoundException("User with identification '" + id + "' not found");
        }

        User user = userRepository.findById(longId)
                .orElseThrow(() -> new UserNotFoundException("User with identification '" + id + "' not found"));
        Hibernate.initialize(user.getGroupMemberships());
        return user;
    }
}
