package cz.neit.dm.test_integration.controllers;

import cz.neit.dm.dtos.modeldto.LoginDTO;
import cz.neit.dm.dtos.modeldto.UserWithPermissionsDTO;
import cz.neit.security.jwt.authentication.utils.JwtTokenUtils;
import cz.neit.security.jwt.authentication.utils.SecurityConstants;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;

import static org.hamcrest.core.StringStartsWith.startsWith;
import static org.junit.Assert.*;


@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("test")
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class LoginControllerTest {

    @Autowired
    TestRestTemplate rest;
    @Autowired
    JwtTokenUtils jwtTokenUtils;

    @Test
    public void test01_CorrectLogin() {
        LoginDTO loginRequest = new LoginDTO();
        loginRequest.setUsername("petrmlady");
        loginRequest.setPassword("admin");

        ResponseEntity<UserWithPermissionsDTO> response = rest.postForEntity("/login", loginRequest, UserWithPermissionsDTO.class);
        UserWithPermissionsDTO body = response.getBody();

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertNotNull(response.getHeaders().get(SecurityConstants.AUTH_HEADER));
        assertEquals(1, response.getHeaders().get(SecurityConstants.AUTH_HEADER).size());
        assertThat(response.getHeaders().get(SecurityConstants.AUTH_HEADER).get(0), startsWith("Bearer "));
        assertEquals("petrmlady", body.getUsername());
        assertNotNull(body.getPermissions());
    }

    @Test
    public void test02_BadLoginUsername() {
        LoginDTO loginRequest = new LoginDTO();
        loginRequest.setUsername("notWorkingUsername");
        loginRequest.setPassword("notWorkingPasswd");

        ResponseEntity<UserWithPermissionsDTO> response = rest.postForEntity("/login", loginRequest, UserWithPermissionsDTO.class);

        assertEquals(HttpStatus.UNAUTHORIZED, response.getStatusCode());
        assertNull(response.getHeaders().get(SecurityConstants.AUTH_HEADER));
    }

    @Test
    public void test02_BadLoginPassword() {
        LoginDTO loginRequest = new LoginDTO();
        loginRequest.setUsername("petrmlady");
        loginRequest.setPassword("notWorkingPasswd");

        ResponseEntity<UserWithPermissionsDTO> response = rest.postForEntity("/login", loginRequest, UserWithPermissionsDTO.class);

        assertEquals(HttpStatus.UNAUTHORIZED, response.getStatusCode());
        assertNull(response.getHeaders().get(SecurityConstants.AUTH_HEADER));
    }
}
