package cz.neit.dm.test_integration.controllers;

import cz.neit.dm.dtos.modeldto.RecipientDTO;
import cz.neit.dm.test_integration.ControllerTestHelper;
import cz.neit.security.jwt.authentication.utils.JwtTokenUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;

import static org.junit.Assert.assertEquals;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("test")
public class RecipientControllerTest {

    @Autowired
    TestRestTemplate rest;

    @Autowired
    JwtTokenUtils jwtTokenUtils;

    @Before
    public void setUp() {
        ControllerTestHelper.setAdminJwtAuthToken(rest.getRestTemplate(), jwtTokenUtils);
    }

    @Test
    public void testGetRecipients_emailNull() {
        ResponseEntity<RecipientDTO[]> response = rest.getForEntity("/recipients", RecipientDTO[].class);
        RecipientDTO[] body = response.getBody();

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(3, body.length);
    }

    @Test
    public void testGetRecipients_emailSearch() {
        ResponseEntity<RecipientDTO[]> response = rest.getForEntity("/recipients?email={email}",
                RecipientDTO[].class, "test-domain.com");
        RecipientDTO[] body = response.getBody();

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(1, body.length);
    }
}
